@@ -4,12 +4,12 @@ Species distribution modeling
 =============================
 
 Modeling species' geographic distributions is an important
-problem in conservation biology. In this example we
-model the geographic distribution of two south american
+problem in conservation biology. In this example, we
+model the geographic distribution of two South American
 mammals given past observations and 14 environmental
 variables. Since we have only positive examples (there are
 no unsuccessful observations), we cast this problem as a
-density estimation problem and use the :class:`sklearn.svm.OneClassSVM`
+density estimation problem and use the :class:`~sklearn.svm.OneClassSVM`
 as our modeling tool. The dataset is provided by Phillips et. al. (2006).
 If available, the example uses
 `basemap <https://matplotlib.org/basemap/>`_
@@ -33,6 +33,7 @@ References
    <http://rob.schapire.net/papers/ecolmod.pdf>`_
    S. J. Phillips, R. P. Anderson, R. E. Schapire - Ecological Modelling,
    190:231-259, 2006.
+
 """
 
 # Authors: Peter Prettenhofer <peter.prettenhofer@gmail.com>
@@ -53,12 +54,11 @@ from sklearn import svm, metrics
 # otherwise, we'll improvise later...
 try:
     from mpl_toolkits.basemap import Basemap
+
     basemap = True
 except ImportError:
     basemap = False
 
-print(__doc__)
-
 
 def construct_grids(batch):
     """Construct the map grid from the batch object
@@ -93,31 +93,34 @@ def create_species_bunch(species_name, train, test, coverages, xgrid, ygrid):
     This will use the test/train record arrays to extract the
     data specific to the given species name.
     """
-    bunch = Bunch(name=' '.join(species_name.split("_")[:2]))
-    species_name = species_name.encode('ascii')
+    bunch = Bunch(name=" ".join(species_name.split("_")[:2]))
+    species_name = species_name.encode("ascii")
     points = dict(test=test, train=train)
 
     for label, pts in points.items():
         # choose points associated with the desired species
-        pts = pts[pts['species'] == species_name]
-        bunch['pts_%s' % label] = pts
+        pts = pts[pts["species"] == species_name]
+        bunch["pts_%s" % label] = pts
 
         # determine coverage values for each of the training & testing points
-        ix = np.searchsorted(xgrid, pts['dd long'])
-        iy = np.searchsorted(ygrid, pts['dd lat'])
-        bunch['cov_%s' % label] = coverages[:, -iy, ix].T
+        ix = np.searchsorted(xgrid, pts["dd long"])
+        iy = np.searchsorted(ygrid, pts["dd lat"])
+        bunch["cov_%s" % label] = coverages[:, -iy, ix].T
 
     return bunch
 
 
-def plot_species_distribution(species=("bradypus_variegatus_0",
-                                       "microryzomys_minutus_0")):
+def plot_species_distribution(
+    species=("bradypus_variegatus_0", "microryzomys_minutus_0")
+):
     """
     Plot the species distribution.
     """
     if len(species) > 2:
-        print("Note: when more than two species are provided,"
-              " only the first two will be used")
+        print(
+            "Note: when more than two species are provided,"
+            " only the first two will be used"
+        )
 
     t0 = time()
 
@@ -131,19 +134,19 @@ def plot_species_distribution(species=("bradypus_variegatus_0",
     X, Y = np.meshgrid(xgrid, ygrid[::-1])
 
     # create a bunch for each species
-    BV_bunch = create_species_bunch(species[0],
-                                    data.train, data.test,
-                                    data.coverages, xgrid, ygrid)
-    MM_bunch = create_species_bunch(species[1],
-                                    data.train, data.test,
-                                    data.coverages, xgrid, ygrid)
+    BV_bunch = create_species_bunch(
+        species[0], data.train, data.test, data.coverages, xgrid, ygrid
+    )
+    MM_bunch = create_species_bunch(
+        species[1], data.train, data.test, data.coverages, xgrid, ygrid
+    )
 
     # background points (grid coordinates) for evaluation
     np.random.seed(13)
-    background_points = np.c_[np.random.randint(low=0, high=data.Ny,
-                                                size=10000),
-                              np.random.randint(low=0, high=data.Nx,
-                                                size=10000)].T
+    background_points = np.c_[
+        np.random.randint(low=0, high=data.Ny, size=10000),
+        np.random.randint(low=0, high=data.Nx, size=10000),
+    ].T
 
     # We'll make use of the fact that coverages[6] has measurements at all
     # land points.  This will help us decide between land and water.
@@ -160,7 +163,7 @@ def plot_species_distribution(species=("bradypus_variegatus_0",
         train_cover_std = (species.cov_train - mean) / std
 
         # Fit OneClassSVM
-        print(" - fit OneClassSVM ... ", end='')
+        print(" - fit OneClassSVM ... ", end="")
         clf = svm.OneClassSVM(nu=0.1, kernel="rbf", gamma=0.5)
         clf.fit(train_cover_std)
         print("done.")
@@ -169,16 +172,21 @@ def plot_species_distribution(species=("bradypus_variegatus_0",
         plt.subplot(1, 2, i + 1)
         if basemap:
             print(" - plot coastlines using basemap")
-            m = Basemap(projection='cyl', llcrnrlat=Y.min(),
-                        urcrnrlat=Y.max(), llcrnrlon=X.min(),
-                        urcrnrlon=X.max(), resolution='c')
+            m = Basemap(
+                projection="cyl",
+                llcrnrlat=Y.min(),
+                urcrnrlat=Y.max(),
+                llcrnrlon=X.min(),
+                urcrnrlon=X.max(),
+                resolution="c",
+            )
             m.drawcoastlines()
             m.drawcountries()
         else:
             print(" - plot coastlines from coverage")
-            plt.contour(X, Y, land_reference,
-                        levels=[-9998], colors="k",
-                        linestyles="solid")
+            plt.contour(
+                X, Y, land_reference, levels=[-9998], colors="k", linestyles="solid"
+            )
             plt.xticks([])
             plt.yticks([])
 
@@ -200,18 +208,28 @@ def plot_species_distribution(species=("bradypus_variegatus_0",
 
         # plot contours of the prediction
         plt.contourf(X, Y, Z, levels=levels, cmap=plt.cm.Reds)
-        plt.colorbar(format='%.2f')
+        plt.colorbar(format="%.2f")
 
         # scatter training/testing points
-        plt.scatter(species.pts_train['dd long'], species.pts_train['dd lat'],
-                    s=2 ** 2, c='black',
-                    marker='^', label='train')
-        plt.scatter(species.pts_test['dd long'], species.pts_test['dd lat'],
-                    s=2 ** 2, c='black',
-                    marker='x', label='test')
+        plt.scatter(
+            species.pts_train["dd long"],
+            species.pts_train["dd lat"],
+            s=2**2,
+            c="black",
+            marker="^",
+            label="train",
+        )
+        plt.scatter(
+            species.pts_test["dd long"],
+            species.pts_test["dd lat"],
+            s=2**2,
+            c="black",
+            marker="x",
+            label="test",
+        )
         plt.legend()
         plt.title(species.name)
-        plt.axis('equal')
+        plt.axis("equal")
 
         # Compute AUC with regards to background points
         pred_background = Z[background_points[0], background_points[1]]
