@@ -13,40 +13,38 @@ negative examples click the right button.
 If all examples are from the same class, it uses a one-class SVM.
 
 """
-from __future__ import division, print_function
-
-print(__doc__)
 
 # Author: Peter Prettenhoer <peter.prettenhofer@gmail.com>
 #
 # License: BSD 3 clause
 
 import matplotlib
-matplotlib.use('TkAgg')
 
+matplotlib.use("TkAgg")
 from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
-from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg
+
+try:
+    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk
+except ImportError:
+    # NavigationToolbar2TkAgg was deprecated in matplotlib 2.2
+    from matplotlib.backends.backend_tkagg import (
+        NavigationToolbar2TkAgg as NavigationToolbar2Tk,
+    )
 from matplotlib.figure import Figure
 from matplotlib.contour import ContourSet
 
-try:
-    import tkinter as Tk
-except ImportError:
-    # Backward compat for Python 2
-    import Tkinter as Tk
-
 import sys
 import numpy as np
+import tkinter as Tk
 
 from sklearn import svm
 from sklearn.datasets import dump_svmlight_file
-from sklearn.externals.six.moves import xrange
 
 y_min, y_max = -50, 50
 x_min, x_max = -50, 50
 
 
-class Model(object):
+class Model:
     """The Model which hold the data. It implements the
     observable in the observer pattern and notifies the
     registered observers on change event.
@@ -60,12 +58,12 @@ class Model(object):
         self.surface_type = 0
 
     def changed(self, event):
-        """Notify the observers. """
+        """Notify the observers."""
         for observer in self.observers:
             observer.update(event, self)
 
     def add_observer(self, observer):
-        """Register an observer. """
+        """Register an observer."""
         self.observers.append(observer)
 
     def set_surface(self, surface):
@@ -78,7 +76,7 @@ class Model(object):
         dump_svmlight_file(X, y, file)
 
 
-class Controller(object):
+class Controller:
     def __init__(self, model):
         self.model = model
         self.kernel = Tk.IntVar()
@@ -98,14 +96,23 @@ class Controller(object):
         degree = int(self.degree.get())
         kernel_map = {0: "linear", 1: "rbf", 2: "poly"}
         if len(np.unique(y)) == 1:
-            clf = svm.OneClassSVM(kernel=kernel_map[self.kernel.get()],
-                                  gamma=gamma, coef0=coef0, degree=degree)
+            clf = svm.OneClassSVM(
+                kernel=kernel_map[self.kernel.get()],
+                gamma=gamma,
+                coef0=coef0,
+                degree=degree,
+            )
             clf.fit(X)
         else:
-            clf = svm.SVC(kernel=kernel_map[self.kernel.get()], C=C,
-                          gamma=gamma, coef0=coef0, degree=degree)
+            clf = svm.SVC(
+                kernel=kernel_map[self.kernel.get()],
+                C=C,
+                gamma=gamma,
+                coef0=coef0,
+                degree=degree,
+            )
             clf.fit(X, y)
-        if hasattr(clf, 'score'):
+        if hasattr(clf, "score"):
             print("Accuracy:", clf.score(X, y) * 100)
         X1, X2, Z = self.decision_surface(clf)
         self.model.clf = clf
@@ -136,13 +143,14 @@ class Controller(object):
         self.refit()
 
     def refit(self):
-        """Refit the model if already fitted. """
+        """Refit the model if already fitted."""
         if self.fitted:
             self.fit()
 
 
-class View(object):
-    """Test docstring. """
+class View:
+    """Test docstring."""
+
     def __init__(self, root, controller):
         f = Figure()
         ax = f.add_subplot(111)
@@ -151,11 +159,15 @@ class View(object):
         ax.set_xlim((x_min, x_max))
         ax.set_ylim((y_min, y_max))
         canvas = FigureCanvasTkAgg(f, master=root)
-        canvas.show()
+        try:
+            canvas.draw()
+        except AttributeError:
+            # support for matplotlib (1.*)
+            canvas.show()
         canvas.get_tk_widget().pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)
         canvas._tkcanvas.pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)
-        canvas.mpl_connect('button_press_event', self.onclick)
-        toolbar = NavigationToolbar2TkAgg(canvas, root)
+        canvas.mpl_connect("button_press_event", self.onclick)
+        toolbar = NavigationToolbar2Tk(canvas, root)
         toolbar.update()
         self.controllbar = ControllBar(root, controller)
         self.f = f
@@ -181,14 +193,14 @@ class View(object):
     def update_example(self, model, idx):
         x, y, l = model.data[idx]
         if l == 1:
-            color = 'w'
+            color = "w"
         elif l == -1:
-            color = 'k'
+            color = "k"
         self.ax.plot([x], [y], "%so" % color, scalex=0.0, scaley=0.0)
 
     def update(self, event, model):
         if event == "examples_loaded":
-            for i in xrange(len(model.data)):
+            for i in range(len(model.data)):
                 self.update_example(model, i)
 
         if event == "example_added":
@@ -224,39 +236,62 @@ class View(object):
         """Plot the support vectors by placing circles over the
         corresponding data points and adds the circle collection
         to the contours list."""
-        cs = self.ax.scatter(support_vectors[:, 0], support_vectors[:, 1],
-                             s=80, edgecolors="k", facecolors="none")
+        cs = self.ax.scatter(
+            support_vectors[:, 0],
+            support_vectors[:, 1],
+            s=80,
+            edgecolors="k",
+            facecolors="none",
+        )
         self.contours.append(cs)
 
     def plot_decision_surface(self, surface, type):
         X1, X2, Z = surface
         if type == 0:
             levels = [-1.0, 0.0, 1.0]
-            linestyles = ['dashed', 'solid', 'dashed']
-            colors = 'k'
-            self.contours.append(self.ax.contour(X1, X2, Z, levels,
-                                                 colors=colors,
-                                                 linestyles=linestyles))
+            linestyles = ["dashed", "solid", "dashed"]
+            colors = "k"
+            self.contours.append(
+                self.ax.contour(X1, X2, Z, levels, colors=colors, linestyles=linestyles)
+            )
         elif type == 1:
-            self.contours.append(self.ax.contourf(X1, X2, Z, 10,
-                                                  cmap=matplotlib.cm.bone,
-                                                  origin='lower', alpha=0.85))
-            self.contours.append(self.ax.contour(X1, X2, Z, [0.0], colors='k',
-                                                 linestyles=['solid']))
+            self.contours.append(
+                self.ax.contourf(
+                    X1, X2, Z, 10, cmap=matplotlib.cm.bone, origin="lower", alpha=0.85
+                )
+            )
+            self.contours.append(
+                self.ax.contour(X1, X2, Z, [0.0], colors="k", linestyles=["solid"])
+            )
         else:
             raise ValueError("surface type unknown")
 
 
-class ControllBar(object):
+class ControllBar:
     def __init__(self, root, controller):
         fm = Tk.Frame(root)
         kernel_group = Tk.Frame(fm)
-        Tk.Radiobutton(kernel_group, text="Linear", variable=controller.kernel,
-                       value=0, command=controller.refit).pack(anchor=Tk.W)
-        Tk.Radiobutton(kernel_group, text="RBF", variable=controller.kernel,
-                       value=1, command=controller.refit).pack(anchor=Tk.W)
-        Tk.Radiobutton(kernel_group, text="Poly", variable=controller.kernel,
-                       value=2, command=controller.refit).pack(anchor=Tk.W)
+        Tk.Radiobutton(
+            kernel_group,
+            text="Linear",
+            variable=controller.kernel,
+            value=0,
+            command=controller.refit,
+        ).pack(anchor=Tk.W)
+        Tk.Radiobutton(
+            kernel_group,
+            text="RBF",
+            variable=controller.kernel,
+            value=1,
+            command=controller.refit,
+        ).pack(anchor=Tk.W)
+        Tk.Radiobutton(
+            kernel_group,
+            text="Poly",
+            variable=controller.kernel,
+            value=2,
+            command=controller.refit,
+        ).pack(anchor=Tk.W)
         kernel_group.pack(side=Tk.LEFT)
 
         valbox = Tk.Frame(fm)
@@ -264,8 +299,7 @@ class ControllBar(object):
         controller.complexity.set("1.0")
         c = Tk.Frame(valbox)
         Tk.Label(c, text="C:", anchor="e", width=7).pack(side=Tk.LEFT)
-        Tk.Entry(c, width=6, textvariable=controller.complexity).pack(
-            side=Tk.LEFT)
+        Tk.Entry(c, width=6, textvariable=controller.complexity).pack(side=Tk.LEFT)
         c.pack()
 
         controller.gamma = Tk.StringVar()
@@ -291,29 +325,42 @@ class ControllBar(object):
         valbox.pack(side=Tk.LEFT)
 
         cmap_group = Tk.Frame(fm)
-        Tk.Radiobutton(cmap_group, text="Hyperplanes",
-                       variable=controller.surface_type, value=0,
-                       command=controller.refit).pack(anchor=Tk.W)
-        Tk.Radiobutton(cmap_group, text="Surface",
-                       variable=controller.surface_type, value=1,
-                       command=controller.refit).pack(anchor=Tk.W)
+        Tk.Radiobutton(
+            cmap_group,
+            text="Hyperplanes",
+            variable=controller.surface_type,
+            value=0,
+            command=controller.refit,
+        ).pack(anchor=Tk.W)
+        Tk.Radiobutton(
+            cmap_group,
+            text="Surface",
+            variable=controller.surface_type,
+            value=1,
+            command=controller.refit,
+        ).pack(anchor=Tk.W)
 
         cmap_group.pack(side=Tk.LEFT)
 
-        train_button = Tk.Button(fm, text='Fit', width=5,
-                                 command=controller.fit)
+        train_button = Tk.Button(fm, text="Fit", width=5, command=controller.fit)
         train_button.pack()
         fm.pack(side=Tk.LEFT)
-        Tk.Button(fm, text='Clear', width=5,
-                  command=controller.clear_data).pack(side=Tk.LEFT)
+        Tk.Button(fm, text="Clear", width=5, command=controller.clear_data).pack(
+            side=Tk.LEFT
+        )
 
 
 def get_parser():
     from optparse import OptionParser
+
     op = OptionParser()
-    op.add_option("--output",
-                  action="store", type="str", dest="output",
-                  help="Path where to dump data.")
+    op.add_option(
+        "--output",
+        action="store",
+        type="str",
+        dest="output",
+        help="Path where to dump data.",
+    )
     return op
 
 
@@ -331,5 +378,6 @@ def main(argv):
     if opts.output:
         model.dump_svmlight_file(opts.output)
 
+
 if __name__ == "__main__":
     main(sys.argv)
