@@ -65,7 +65,8 @@ for url, filename in resources:
     if not os.path.exists(filename):
         print("Downloading data from '%s', please wait..." % url)
         opener = urlopen(url)
-        open(filename, "wb").write(opener.read())
+        with open(filename, "wb") as f:
+            f.write(opener.read())
         print()
 
 
