@@ -62,11 +62,11 @@ bdt_discrete.fit(X_train, y_train)
 real_test_errors = []
 discrete_test_errors = []
 
-for real_test_predict, discrete_train_predict in zip(
+for real_test_predict, discrete_test_predict in zip(
     bdt_real.staged_predict(X_test), bdt_discrete.staged_predict(X_test)
 ):
     real_test_errors.append(1.0 - accuracy_score(real_test_predict, y_test))
-    discrete_test_errors.append(1.0 - accuracy_score(discrete_train_predict, y_test))
+    discrete_test_errors.append(1.0 - accuracy_score(discrete_test_predict, y_test))
 
 n_trees_discrete = len(bdt_discrete)
 n_trees_real = len(bdt_real)
