@@ -29,15 +29,18 @@ until the validation score did not improve by at least ``tol`` during the last
 at the attribute ``n_iter_``.
 
 This example illustrates how the early stopping can used in the
-:class:`sklearn.linear_model.SGDClassifier` model to achieve almost the same
+:class:`~sklearn.linear_model.SGDClassifier` model to achieve almost the same
 accuracy as compared to a model built without early stopping. This can
 significantly reduce training time. Note that scores differ between the
 stopping criteria even from early iterations because some of the training data
 is held out with the validation stopping criterion.
+
 """
+
 # Authors: Tom Dupre la Tour
 #
 # License: BSD 3 clause
+
 import time
 import sys
 
@@ -52,13 +55,11 @@ from sklearn.utils._testing import ignore_warnings
 from sklearn.exceptions import ConvergenceWarning
 from sklearn.utils import shuffle
 
-print(__doc__)
 
-
-def load_mnist(n_samples=None, class_0='0', class_1='8'):
+def load_mnist(n_samples=None, class_0="0", class_1="8"):
     """Load MNIST, select two classes, shuffle and return only n_samples."""
     # Load data from http://openml.org/d/554
-    mnist = fetch_openml('mnist_784', version=1)
+    mnist = fetch_openml("mnist_784", version=1, as_frame=False, parser="pandas")
 
     # take only two classes for binary classification
     mask = np.logical_or(mnist.target == class_0, mnist.target == class_1)
@@ -88,62 +89,67 @@ def fit_and_score(estimator, max_iter, X_train, X_test, y_train, y_test):
 
 # Define the estimators to compare
 estimator_dict = {
-    'No stopping criterion':
-    linear_model.SGDClassifier(n_iter_no_change=3),
-    'Training loss':
-    linear_model.SGDClassifier(early_stopping=False, n_iter_no_change=3,
-                               tol=0.1),
-    'Validation score':
-    linear_model.SGDClassifier(early_stopping=True, n_iter_no_change=3,
-                               tol=0.0001, validation_fraction=0.2)
+    "No stopping criterion": linear_model.SGDClassifier(n_iter_no_change=3),
+    "Training loss": linear_model.SGDClassifier(
+        early_stopping=False, n_iter_no_change=3, tol=0.1
+    ),
+    "Validation score": linear_model.SGDClassifier(
+        early_stopping=True, n_iter_no_change=3, tol=0.0001, validation_fraction=0.2
+    ),
 }
 
 # Load the dataset
 X, y = load_mnist(n_samples=10000)
-X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.5,
-                                                    random_state=0)
+X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.5, random_state=0)
 
 results = []
 for estimator_name, estimator in estimator_dict.items():
-    print(estimator_name + ': ', end='')
+    print(estimator_name + ": ", end="")
     for max_iter in range(1, 50):
-        print('.', end='')
+        print(".", end="")
         sys.stdout.flush()
 
         fit_time, n_iter, train_score, test_score = fit_and_score(
-            estimator, max_iter, X_train, X_test, y_train, y_test)
+            estimator, max_iter, X_train, X_test, y_train, y_test
+        )
 
-        results.append((estimator_name, max_iter, fit_time, n_iter,
-                        train_score, test_score))
-    print('')
+        results.append(
+            (estimator_name, max_iter, fit_time, n_iter, train_score, test_score)
+        )
+    print("")
 
 # Transform the results in a pandas dataframe for easy plotting
 columns = [
-    'Stopping criterion', 'max_iter', 'Fit time (sec)', 'n_iter_',
-    'Train score', 'Test score'
+    "Stopping criterion",
+    "max_iter",
+    "Fit time (sec)",
+    "n_iter_",
+    "Train score",
+    "Test score",
 ]
 results_df = pd.DataFrame(results, columns=columns)
 
-# Define what to plot (x_axis, y_axis)
-lines = 'Stopping criterion'
-plot_list = [
-    ('max_iter', 'Train score'),
-    ('max_iter', 'Test score'),
-    ('max_iter', 'n_iter_'),
-    ('max_iter', 'Fit time (sec)'),
-]
+# Define what to plot
+lines = "Stopping criterion"
+x_axis = "max_iter"
+styles = ["-.", "--", "-"]
 
-nrows = 2
-ncols = int(np.ceil(len(plot_list) / 2.))
-fig, axes = plt.subplots(nrows=nrows, ncols=ncols, figsize=(6 * ncols,
-                                                            4 * nrows))
-axes[0, 0].get_shared_y_axes().join(axes[0, 0], axes[0, 1])
-
-for ax, (x_axis, y_axis) in zip(axes.ravel(), plot_list):
-    for criterion, group_df in results_df.groupby(lines):
-        group_df.plot(x=x_axis, y=y_axis, label=criterion, ax=ax)
+# First plot: train and test scores
+fig, axes = plt.subplots(nrows=1, ncols=2, sharey=True, figsize=(12, 4))
+for ax, y_axis in zip(axes, ["Train score", "Test score"]):
+    for style, (criterion, group_df) in zip(styles, results_df.groupby(lines)):
+        group_df.plot(x=x_axis, y=y_axis, label=criterion, ax=ax, style=style)
     ax.set_title(y_axis)
     ax.legend(title=lines)
-
 fig.tight_layout()
+
+# Second plot: n_iter and fit time
+fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(12, 4))
+for ax, y_axis in zip(axes, ["n_iter_", "Fit time (sec)"]):
+    for style, (criterion, group_df) in zip(styles, results_df.groupby(lines)):
+        group_df.plot(x=x_axis, y=y_axis, label=criterion, ax=ax, style=style)
+    ax.set_title(y_axis)
+    ax.legend(title=lines)
+fig.tight_layout()
+
 plt.show()
