@@ -8,34 +8,36 @@ with RBF kernel. The target to predict is a XOR of the
 inputs.
 
 The color map illustrates the decision function learned by the SVC.
+
 """
-print(__doc__)
 
 import numpy as np
 import matplotlib.pyplot as plt
 from sklearn import svm
 
-xx, yy = np.meshgrid(np.linspace(-3, 3, 500),
-                     np.linspace(-3, 3, 500))
+xx, yy = np.meshgrid(np.linspace(-3, 3, 500), np.linspace(-3, 3, 500))
 np.random.seed(0)
 X = np.random.randn(300, 2)
 Y = np.logical_xor(X[:, 0] > 0, X[:, 1] > 0)
 
 # fit the model
-clf = svm.NuSVC(gamma='auto')
+clf = svm.NuSVC(gamma="auto")
 clf.fit(X, Y)
 
 # plot the decision function for each datapoint on the grid
 Z = clf.decision_function(np.c_[xx.ravel(), yy.ravel()])
 Z = Z.reshape(xx.shape)
 
-plt.imshow(Z, interpolation='nearest',
-           extent=(xx.min(), xx.max(), yy.min(), yy.max()), aspect='auto',
-           origin='lower', cmap=plt.cm.PuOr_r)
-contours = plt.contour(xx, yy, Z, levels=[0], linewidths=2,
-                       linestyles='dashed')
-plt.scatter(X[:, 0], X[:, 1], s=30, c=Y, cmap=plt.cm.Paired,
-            edgecolors='k')
+plt.imshow(
+    Z,
+    interpolation="nearest",
+    extent=(xx.min(), xx.max(), yy.min(), yy.max()),
+    aspect="auto",
+    origin="lower",
+    cmap=plt.cm.PuOr_r,
+)
+contours = plt.contour(xx, yy, Z, levels=[0], linewidths=2, linestyles="dashed")
+plt.scatter(X[:, 0], X[:, 1], s=30, c=Y, cmap=plt.cm.Paired, edgecolors="k")
 plt.xticks(())
 plt.yticks(())
 plt.axis([-3, 3, -3, 3])
