@@ -16,18 +16,20 @@ Seeding is performed using a binning technique for scalability.
 
 import numpy as np
 import warnings
-from joblib import Parallel, delayed
+from numbers import Integral, Real
 
 from collections import defaultdict
+from ..utils._param_validation import Interval
 from ..utils.validation import check_is_fitted
+from ..utils.parallel import delayed, Parallel
 from ..utils import check_random_state, gen_batches, check_array
 from ..base import BaseEstimator, ClusterMixin
 from ..neighbors import NearestNeighbors
 from ..metrics.pairwise import pairwise_distances_argmin
+from .._config import config_context
 
 
-def estimate_bandwidth(X, quantile=0.3, n_samples=None, random_state=0,
-                       n_jobs=None):
+def estimate_bandwidth(X, *, quantile=0.3, n_samples=None, random_state=0, n_jobs=None):
     """Estimate the bandwidth to use with the mean-shift algorithm.
 
     That this function takes time at least quadratic in n_samples. For large
@@ -38,20 +40,20 @@ def estimate_bandwidth(X, quantile=0.3, n_samples=None, random_state=0,
     X : array-like of shape (n_samples, n_features)
         Input points.
 
-    quantile : float, default 0.3
-        should be between [0, 1]
+    quantile : float, default=0.3
+        Should be between [0, 1]
         0.5 means that the median of all pairwise distances is used.
 
-    n_samples : int, optional
+    n_samples : int, default=None
         The number of samples to use. If not given, all samples are used.
 
-    random_state : int, RandomState instance or None (default)
+    random_state : int, RandomState instance, default=None
         The generator used to randomly select the samples from input points
         for bandwidth estimation. Use an int to make the randomness
         deterministic.
         See :term:`Glossary <random_state>`.
 
-    n_jobs : int or None, optional (default=None)
+    n_jobs : int, default=None
         The number of parallel jobs to run for neighbors search.
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
@@ -71,11 +73,10 @@ def estimate_bandwidth(X, quantile=0.3, n_samples=None, random_state=0,
     n_neighbors = int(X.shape[0] * quantile)
     if n_neighbors < 1:  # cannot fit NearestNeighbors with n_neighbors = 0
         n_neighbors = 1
-    nbrs = NearestNeighbors(n_neighbors=n_neighbors,
-                            n_jobs=n_jobs)
+    nbrs = NearestNeighbors(n_neighbors=n_neighbors, n_jobs=n_jobs)
     nbrs.fit(X)
 
-    bandwidth = 0.
+    bandwidth = 0.0
     for batch in gen_batches(len(X), 500):
         d, _ = nbrs.kneighbors(X[batch, :], return_distance=True)
         bandwidth += np.max(d, axis=1).sum()
@@ -86,29 +87,38 @@ def estimate_bandwidth(X, quantile=0.3, n_samples=None, random_state=0,
 # separate function for each seed's iterative loop
 def _mean_shift_single_seed(my_mean, X, nbrs, max_iter):
     # For each seed, climb gradient until convergence or max_iter
-    bandwidth = nbrs.get_params()['radius']
+    bandwidth = nbrs.get_params()["radius"]
     stop_thresh = 1e-3 * bandwidth  # when mean has converged
     completed_iterations = 0
     while True:
         # Find mean of points within bandwidth
-        i_nbrs = nbrs.radius_neighbors([my_mean], bandwidth,
-                                       return_distance=False)[0]
+        i_nbrs = nbrs.radius_neighbors([my_mean], bandwidth, return_distance=False)[0]
         points_within = X[i_nbrs]
         if len(points_within) == 0:
             break  # Depending on seeding strategy this condition may occur
         my_old_mean = my_mean  # save the old mean
         my_mean = np.mean(points_within, axis=0)
         # If converged or at max_iter, adds the cluster
-        if (np.linalg.norm(my_mean - my_old_mean) < stop_thresh or
-                completed_iterations == max_iter):
+        if (
+            np.linalg.norm(my_mean - my_old_mean) < stop_thresh
+            or completed_iterations == max_iter
+        ):
             break
         completed_iterations += 1
     return tuple(my_mean), len(points_within), completed_iterations
 
 
-def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
-               min_bin_freq=1, cluster_all=True, max_iter=300,
-               n_jobs=None):
+def mean_shift(
+    X,
+    *,
+    bandwidth=None,
+    seeds=None,
+    bin_seeding=False,
+    min_bin_freq=1,
+    cluster_all=True,
+    max_iter=300,
+    n_jobs=None,
+):
     """Perform mean shift clustering of data using a flat kernel.
 
     Read more in the :ref:`User Guide <mean_shift>`.
@@ -119,7 +129,7 @@ def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
     X : array-like of shape (n_samples, n_features)
         Input data.
 
-    bandwidth : float, optional
+    bandwidth : float, default=None
         Kernel bandwidth.
 
         If bandwidth is not given, it is determined using a heuristic based on
@@ -132,7 +142,7 @@ def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
         each data point is used as a seed. If None and bin_seeding=True,
         see bin_seeding.
 
-    bin_seeding : boolean, default=False
+    bin_seeding : bool, default=False
         If true, initial kernel locations are not locations of all
         points, but rather the location of the discretized version of
         points, where points are binned onto a grid whose coarseness
@@ -144,18 +154,25 @@ def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
        To speed up the algorithm, accept only those bins with at least
        min_bin_freq points as seeds.
 
-    cluster_all : boolean, default True
+    cluster_all : bool, default=True
         If true, then all points are clustered, even those orphans that are
         not within any kernel. Orphans are assigned to the nearest kernel.
         If false, then orphans are given cluster label -1.
 
-    max_iter : int, default 300
+    max_iter : int, default=300
         Maximum number of iterations, per seed point before the clustering
         operation terminates (for that seed point), if has not converged yet.
 
-    n_jobs : int or None, optional (default=None)
-        The number of jobs to use for the computation. This works by computing
-        each of the n_init runs in parallel.
+    n_jobs : int, default=None
+        The number of jobs to use for the computation. The following tasks benefit
+        from the parallelization:
+
+        - The search of nearest neighbors for bandwidth estimation and label
+          assignments. See the details in the docstring of the
+          ``NearestNeighbors`` class.
+        - Hill-climbing optimization for all seeds.
+
+        See :term:`Glossary <n_jobs>` for more details.
 
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
@@ -167,28 +184,31 @@ def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
     Returns
     -------
 
-    cluster_centers : array, shape=[n_clusters, n_features]
+    cluster_centers : ndarray of shape (n_clusters, n_features)
         Coordinates of cluster centers.
 
-    labels : array, shape=[n_samples]
+    labels : ndarray of shape (n_samples,)
         Cluster labels for each point.
 
     Notes
     -----
     For an example, see :ref:`examples/cluster/plot_mean_shift.py
     <sphx_glr_auto_examples_cluster_plot_mean_shift.py>`.
-
     """
-    model = MeanShift(bandwidth=bandwidth, seeds=seeds,
-                      min_bin_freq=min_bin_freq,
-                      bin_seeding=bin_seeding,
-                      cluster_all=cluster_all, n_jobs=n_jobs,
-                      max_iter=max_iter).fit(X)
+    model = MeanShift(
+        bandwidth=bandwidth,
+        seeds=seeds,
+        min_bin_freq=min_bin_freq,
+        bin_seeding=bin_seeding,
+        cluster_all=cluster_all,
+        n_jobs=n_jobs,
+        max_iter=max_iter,
+    ).fit(X)
     return model.cluster_centers_, model.labels_
 
 
 def get_bin_seeds(X, bin_size, min_bin_freq=1):
-    """Finds seeds for mean_shift.
+    """Find seeds for mean_shift.
 
     Finds seeds by first binning data onto a grid whose lines are
     spaced bin_size apart, and then choosing those bins with at least
@@ -206,7 +226,7 @@ def get_bin_seeds(X, bin_size, min_bin_freq=1):
         not sure how to set this, set it to the value of the bandwidth used
         in clustering.mean_shift.
 
-    min_bin_freq : integer, optional
+    min_bin_freq : int, default=1
         Only bins with at least min_bin_freq will be selected as seeds.
         Raising this value decreases the number of seeds found, which
         makes mean_shift computationally cheaper.
@@ -216,6 +236,8 @@ def get_bin_seeds(X, bin_size, min_bin_freq=1):
     bin_seeds : array-like of shape (n_samples, n_features)
         Points used as initial kernel positions in clustering.mean_shift.
     """
+    if bin_size == 0:
+        return X
 
     # Bin points
     bin_sizes = defaultdict(int)
@@ -224,11 +246,15 @@ def get_bin_seeds(X, bin_size, min_bin_freq=1):
         bin_sizes[tuple(binned_point)] += 1
 
     # Select only those bins as seeds which have enough members
-    bin_seeds = np.array([point for point, freq in bin_sizes.items() if
-                          freq >= min_bin_freq], dtype=np.float32)
+    bin_seeds = np.array(
+        [point for point, freq in bin_sizes.items() if freq >= min_bin_freq],
+        dtype=np.float32,
+    )
     if len(bin_seeds) == len(X):
-        warnings.warn("Binning data failed with provided bin_size=%f,"
-                      " using data points as seeds." % bin_size)
+        warnings.warn(
+            "Binning data failed with provided bin_size=%f, using data points as seeds."
+            % bin_size
+        )
         return X
     bin_seeds = bin_seeds * bin_size
     return bin_seeds
@@ -249,40 +275,47 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
     Parameters
     ----------
-    bandwidth : float, optional
+    bandwidth : float, default=None
         Bandwidth used in the RBF kernel.
 
         If not given, the bandwidth is estimated using
         sklearn.cluster.estimate_bandwidth; see the documentation for that
         function for hints on scalability (see also the Notes, below).
 
-    seeds : array, shape=[n_samples, n_features], optional
+    seeds : array-like of shape (n_samples, n_features), default=None
         Seeds used to initialize kernels. If not set,
         the seeds are calculated by clustering.get_bin_seeds
         with bandwidth as the grid size and default values for
         other parameters.
 
-    bin_seeding : boolean, optional
+    bin_seeding : bool, default=False
         If true, initial kernel locations are not locations of all
         points, but rather the location of the discretized version of
         points, where points are binned onto a grid whose coarseness
         corresponds to the bandwidth. Setting this option to True will speed
         up the algorithm because fewer seeds will be initialized.
-        default value: False
+        The default value is False.
         Ignored if seeds argument is not None.
 
-    min_bin_freq : int, optional
+    min_bin_freq : int, default=1
        To speed up the algorithm, accept only those bins with at least
-       min_bin_freq points as seeds. If not defined, set to 1.
+       min_bin_freq points as seeds.
 
-    cluster_all : boolean, default True
+    cluster_all : bool, default=True
         If true, then all points are clustered, even those orphans that are
         not within any kernel. Orphans are assigned to the nearest kernel.
         If false, then orphans are given cluster label -1.
 
-    n_jobs : int or None, optional (default=None)
-        The number of jobs to use for the computation. This works by computing
-        each of the n_init runs in parallel.
+    n_jobs : int, default=None
+        The number of jobs to use for the computation. The following tasks benefit
+        from the parallelization:
+
+        - The search of nearest neighbors for bandwidth estimation and label
+          assignments. See the details in the docstring of the
+          ``NearestNeighbors`` class.
+        - Hill-climbing optimization for all seeds.
+
+        See :term:`Glossary <n_jobs>` for more details.
 
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
@@ -296,10 +329,10 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
     Attributes
     ----------
-    cluster_centers_ : array, [n_clusters, n_features]
+    cluster_centers_ : ndarray of shape (n_clusters, n_features)
         Coordinates of cluster centers.
 
-    labels_ :
+    labels_ : ndarray of shape (n_samples,)
         Labels of each point.
 
     n_iter_ : int
@@ -307,19 +340,20 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
         .. versionadded:: 0.22
 
-    Examples
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    See Also
     --------
-    >>> from sklearn.cluster import MeanShift
-    >>> import numpy as np
-    >>> X = np.array([[1, 1], [2, 1], [1, 0],
-    ...               [4, 7], [3, 5], [3, 6]])
-    >>> clustering = MeanShift(bandwidth=2).fit(X)
-    >>> clustering.labels_
-    array([1, 1, 1, 0, 0, 0])
-    >>> clustering.predict([[0, 0], [5, 5]])
-    array([1, 0])
-    >>> clustering
-    MeanShift(bandwidth=2)
+    KMeans : K-Means clustering.
 
     Notes
     -----
@@ -345,9 +379,42 @@ class MeanShift(ClusterMixin, BaseEstimator):
     feature space analysis". IEEE Transactions on Pattern Analysis and
     Machine Intelligence. 2002. pp. 603-619.
 
+    Examples
+    --------
+    >>> from sklearn.cluster import MeanShift
+    >>> import numpy as np
+    >>> X = np.array([[1, 1], [2, 1], [1, 0],
+    ...               [4, 7], [3, 5], [3, 6]])
+    >>> clustering = MeanShift(bandwidth=2).fit(X)
+    >>> clustering.labels_
+    array([1, 1, 1, 0, 0, 0])
+    >>> clustering.predict([[0, 0], [5, 5]])
+    array([1, 0])
+    >>> clustering
+    MeanShift(bandwidth=2)
     """
-    def __init__(self, bandwidth=None, seeds=None, bin_seeding=False,
-                 min_bin_freq=1, cluster_all=True, n_jobs=None, max_iter=300):
+
+    _parameter_constraints: dict = {
+        "bandwidth": [Interval(Real, 0, None, closed="neither"), None],
+        "seeds": ["array-like", None],
+        "bin_seeding": ["boolean"],
+        "min_bin_freq": [Interval(Integral, 1, None, closed="left")],
+        "cluster_all": ["boolean"],
+        "n_jobs": [Integral, None],
+        "max_iter": [Interval(Integral, 0, None, closed="left")],
+    }
+
+    def __init__(
+        self,
+        *,
+        bandwidth=None,
+        seeds=None,
+        bin_seeding=False,
+        min_bin_freq=1,
+        cluster_all=True,
+        n_jobs=None,
+        max_iter=300,
+    ):
         self.bandwidth = bandwidth
         self.seeds = seeds
         self.bin_seeding = bin_seeding
@@ -365,15 +432,18 @@ class MeanShift(ClusterMixin, BaseEstimator):
             Samples to cluster.
 
         y : Ignored
+            Not used, present for API consistency by convention.
 
+        Returns
+        -------
+        self : object
+               Fitted instance.
         """
-        X = check_array(X)
+        self._validate_params()
+        X = self._validate_data(X)
         bandwidth = self.bandwidth
         if bandwidth is None:
             bandwidth = estimate_bandwidth(X, n_jobs=self.n_jobs)
-        elif bandwidth <= 0:
-            raise ValueError("bandwidth needs to be greater than zero or None,"
-                             " got %f" % bandwidth)
 
         seeds = self.seeds
         if seeds is None:
@@ -391,8 +461,9 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
         # execute iterations on all seeds in parallel
         all_res = Parallel(n_jobs=self.n_jobs)(
-            delayed(_mean_shift_single_seed)
-            (seed, X, nbrs, self.max_iter) for seed in seeds)
+            delayed(_mean_shift_single_seed)(seed, X, nbrs, self.max_iter)
+            for seed in seeds
+        )
         # copy results in a dictionary
         for i in range(len(seeds)):
             if all_res[i][1]:  # i.e. len(points_within) > 0
@@ -402,35 +473,39 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
         if not center_intensity_dict:
             # nothing near seeds
-            raise ValueError("No point was within bandwidth=%f of any seed."
-                             " Try a different seeding strategy \
-                             or increase the bandwidth."
-                             % bandwidth)
+            raise ValueError(
+                "No point was within bandwidth=%f of any seed. Try a different seeding"
+                " strategy                              or increase the bandwidth."
+                % bandwidth
+            )
 
         # POST PROCESSING: remove near duplicate points
         # If the distance between two kernels is less than the bandwidth,
         # then we have to remove one because it is a duplicate. Remove the
         # one with fewer points.
 
-        sorted_by_intensity = sorted(center_intensity_dict.items(),
-                                     key=lambda tup: (tup[1], tup[0]),
-                                     reverse=True)
+        sorted_by_intensity = sorted(
+            center_intensity_dict.items(),
+            key=lambda tup: (tup[1], tup[0]),
+            reverse=True,
+        )
         sorted_centers = np.array([tup[0] for tup in sorted_by_intensity])
-        unique = np.ones(len(sorted_centers), dtype=np.bool)
-        nbrs = NearestNeighbors(radius=bandwidth,
-                                n_jobs=self.n_jobs).fit(sorted_centers)
+        unique = np.ones(len(sorted_centers), dtype=bool)
+        nbrs = NearestNeighbors(radius=bandwidth, n_jobs=self.n_jobs).fit(
+            sorted_centers
+        )
         for i, center in enumerate(sorted_centers):
             if unique[i]:
-                neighbor_idxs = nbrs.radius_neighbors([center],
-                                                      return_distance=False)[0]
+                neighbor_idxs = nbrs.radius_neighbors([center], return_distance=False)[
+                    0
+                ]
                 unique[neighbor_idxs] = 0
                 unique[i] = 1  # leave the current point as unique
         cluster_centers = sorted_centers[unique]
 
         # ASSIGN LABELS: a point belongs to the cluster that it is closest to
-        nbrs = NearestNeighbors(n_neighbors=1,
-                                n_jobs=self.n_jobs).fit(cluster_centers)
-        labels = np.zeros(n_samples, dtype=np.int)
+        nbrs = NearestNeighbors(n_neighbors=1, n_jobs=self.n_jobs).fit(cluster_centers)
+        labels = np.zeros(n_samples, dtype=int)
         distances, idxs = nbrs.kneighbors(X)
         if self.cluster_all:
             labels = idxs.flatten()
@@ -447,14 +522,15 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape=[n_samples, n_features]
+        X : array-like of shape (n_samples, n_features)
             New data to predict.
 
         Returns
         -------
-        labels : array, shape [n_samples,]
+        labels : ndarray of shape (n_samples,)
             Index of the cluster each sample belongs to.
         """
         check_is_fitted(self)
-
-        return pairwise_distances_argmin(X, self.cluster_centers_)
+        X = self._validate_data(X, reset=False)
+        with config_context(assume_finite=True):
+            return pairwise_distances_argmin(X, self.cluster_centers_)
