@@ -7,16 +7,16 @@
 """Recursive feature elimination for feature ranking"""
 
 import numpy as np
-import numbers
-from joblib import Parallel, effective_n_jobs
+from numbers import Integral, Real
+from joblib import effective_n_jobs
 
 
 from ..utils.metaestimators import available_if
 from ..utils.metaestimators import _safe_split
+from ..utils._param_validation import HasMethods, Interval
 from ..utils._tags import _safe_tags
 from ..utils.validation import check_is_fitted
-from ..utils.fixes import delayed
-from ..utils.deprecation import deprecated
+from ..utils.parallel import delayed, Parallel
 from ..base import BaseEstimator
 from ..base import MetaEstimatorMixin
 from ..base import clone
@@ -183,6 +183,21 @@ class RFE(SelectorMixin, MetaEstimatorMixin, BaseEstimator):
     array([1, 1, 1, 1, 1, 6, 4, 3, 2, 5])
     """
 
+    _parameter_constraints: dict = {
+        "estimator": [HasMethods(["fit"])],
+        "n_features_to_select": [
+            None,
+            Interval(Real, 0, 1, closed="right"),
+            Interval(Integral, 0, None, closed="neither"),
+        ],
+        "step": [
+            Interval(Integral, 0, None, closed="neither"),
+            Interval(Real, 0, 1, closed="neither"),
+        ],
+        "verbose": ["verbose"],
+        "importance_getter": [str, callable],
+    }
+
     def __init__(
         self,
         estimator,
@@ -232,6 +247,7 @@ class RFE(SelectorMixin, MetaEstimatorMixin, BaseEstimator):
         self : object
             Fitted estimator.
         """
+        self._validate_params()
         return self._fit(X, y, **fit_params)
 
     def _fit(self, X, y, step_score=None, **fit_params):
@@ -249,24 +265,13 @@ class RFE(SelectorMixin, MetaEstimatorMixin, BaseEstimator):
             force_all_finite=not tags.get("allow_nan", True),
             multi_output=True,
         )
-        error_msg = (
-            "n_features_to_select must be either None, a "
-            "positive integer representing the absolute "
-            "number of features or a float in (0.0, 1.0] "
-            "representing a percentage of features to "
-            f"select. Got {self.n_features_to_select}"
-        )
 
         # Initialization
         n_features = X.shape[1]
         if self.n_features_to_select is None:
             n_features_to_select = n_features // 2
-        elif self.n_features_to_select < 0:
-            raise ValueError(error_msg)
-        elif isinstance(self.n_features_to_select, numbers.Integral):  # int
+        elif isinstance(self.n_features_to_select, Integral):  # int
             n_features_to_select = self.n_features_to_select
-        elif self.n_features_to_select > 1.0:  # float > 1
-            raise ValueError(error_msg)
         else:  # float
             n_features_to_select = int(n_features * self.n_features_to_select)
 
@@ -274,8 +279,6 @@ class RFE(SelectorMixin, MetaEstimatorMixin, BaseEstimator):
             step = int(max(1, self.step * n_features))
         else:
             step = int(self.step)
-        if step <= 0:
-            raise ValueError("Step must be >0")
 
         support_ = np.ones(n_features, dtype=bool)
         ranking_ = np.ones(n_features, dtype=int)
@@ -538,15 +541,6 @@ class RFECV(RFE):
     estimator_ : ``Estimator`` instance
         The fitted estimator used to select features.
 
-    grid_scores_ : ndarray of shape (n_subsets_of_features,)
-        The cross-validation scores such that
-        ``grid_scores_[i]`` corresponds to
-        the CV score of the i-th subset of features.
-
-        .. deprecated:: 1.0
-            The `grid_scores_` attribute is deprecated in version 1.0 in favor
-            of `cv_results_` and will be removed in version 1.2.
-
     cv_results_ : dict of ndarrays
         A dict with keys:
 
@@ -592,7 +586,7 @@ class RFECV(RFE):
 
     Notes
     -----
-    The size of ``grid_scores_`` is equal to
+    The size of all values in ``cv_results_`` is equal to
     ``ceil((n_features - min_features_to_select) / step) + 1``,
     where step is the number of features removed at each iteration.
 
@@ -624,6 +618,15 @@ class RFECV(RFE):
     array([1, 1, 1, 1, 1, 6, 4, 3, 2, 5])
     """
 
+    _parameter_constraints: dict = {
+        **RFE._parameter_constraints,
+        "min_features_to_select": [Interval(Integral, 0, None, closed="neither")],
+        "cv": ["cv_object"],
+        "scoring": [None, str, callable],
+        "n_jobs": [None, Integral],
+    }
+    _parameter_constraints.pop("n_features_to_select")
+
     def __init__(
         self,
         estimator,
@@ -670,6 +673,7 @@ class RFECV(RFE):
         self : object
             Fitted estimator.
         """
+        self._validate_params()
         tags = self._get_tags()
         X, y = self._validate_data(
             X,
@@ -689,8 +693,6 @@ class RFECV(RFE):
             step = int(max(1, self.step * n_features))
         else:
             step = int(self.step)
-        if step <= 0:
-            raise ValueError("Step must be >0")
 
         # Build an RFE object, which will evaluate and score each possible
         # feature count, down to self.min_features_to_select
@@ -761,17 +763,3 @@ class RFECV(RFE):
             self.cv_results_[f"split{i}_test_score"] = scores_rev[i]
 
         return self
-
-    # TODO: Remove in v1.2 when grid_scores_ is removed
-    # mypy error: Decorated property not supported
-    @deprecated(  # type: ignore
-        "The `grid_scores_` attribute is deprecated in version 1.0 in favor "
-        "of `cv_results_` and will be removed in version 1.2."
-    )
-    @property
-    def grid_scores_(self):
-        # remove 2 for mean_test_score, std_test_score
-        grid_size = len(self.cv_results_) - 2
-        return np.asarray(
-            [self.cv_results_[f"split{i}_test_score"] for i in range(grid_size)]
-        ).T
