@@ -1,11 +1,10 @@
 import time
 from concurrent.futures import ThreadPoolExecutor
 
-from joblib import Parallel
 import pytest
 
 from sklearn import get_config, set_config, config_context
-from sklearn.utils.fixes import delayed
+from sklearn.utils.parallel import delayed, Parallel
 
 
 def test_config_context():
@@ -14,8 +13,10 @@ def test_config_context():
         "working_memory": 1024,
         "print_changed_only": True,
         "display": "diagram",
+        "array_api_dispatch": False,
         "pairwise_dist_chunk_size": 256,
         "enable_cython_pairwise_dist": True,
+        "transform_output": "default",
     }
 
     # Not using as a context manager affects nothing
@@ -28,8 +29,10 @@ def test_config_context():
             "working_memory": 1024,
             "print_changed_only": True,
             "display": "diagram",
+            "array_api_dispatch": False,
             "pairwise_dist_chunk_size": 256,
             "enable_cython_pairwise_dist": True,
+            "transform_output": "default",
         }
     assert get_config()["assume_finite"] is False
 
@@ -59,8 +62,10 @@ def test_config_context():
         "working_memory": 1024,
         "print_changed_only": True,
         "display": "diagram",
+        "array_api_dispatch": False,
         "pairwise_dist_chunk_size": 256,
         "enable_cython_pairwise_dist": True,
+        "transform_output": "default",
     }
 
     # No positional arguments
@@ -114,15 +119,15 @@ def test_config_threadsafe_joblib(backend):
     should be the same as the value passed to the function. In other words,
     it is not influenced by the other job setting assume_finite to True.
     """
-    assume_finites = [False, True]
-    sleep_durations = [0.1, 0.2]
+    assume_finites = [False, True, False, True]
+    sleep_durations = [0.1, 0.2, 0.1, 0.2]
 
     items = Parallel(backend=backend, n_jobs=2)(
         delayed(set_assume_finite)(assume_finite, sleep_dur)
         for assume_finite, sleep_dur in zip(assume_finites, sleep_durations)
     )
 
-    assert items == [False, True]
+    assert items == [False, True, False, True]
 
 
 def test_config_threadsafe():
@@ -130,8 +135,8 @@ def test_config_threadsafe():
     between threads. Same test as `test_config_threadsafe_joblib` but with
     `ThreadPoolExecutor`."""
 
-    assume_finites = [False, True]
-    sleep_durations = [0.1, 0.2]
+    assume_finites = [False, True, False, True]
+    sleep_durations = [0.1, 0.2, 0.1, 0.2]
 
     with ThreadPoolExecutor(max_workers=2) as e:
         items = [
@@ -139,4 +144,4 @@ def test_config_threadsafe():
             for output in e.map(set_assume_finite, assume_finites, sleep_durations)
         ]
 
-    assert items == [False, True]
+    assert items == [False, True, False, True]
